// Copyright 2018-2023 the Deno authors. All rights reserved. MIT license.
/** This module is browser compatible. */
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _BinaryHeap_data;
import { descend } from "./_comparators.js";
export * from "./_comparators.js";
/** Swaps the values at two indexes in an array. */
function swap(array, a, b) {
    const temp = array[a];
    array[a] = array[b];
    array[b] = temp;
}
/** Returns the parent index for a child index. */
function getParentIndex(index) {
    return Math.floor((index + 1) / 2) - 1;
}
/**
 * A priority queue implemented with a binary heap. The heap is in descending
 * order by default, using JavaScript's built-in comparison operators to sort
 * the values.
 *
 * | Method      | Average Case | Worst Case |
 * | ----------- | ------------ | ---------- |
 * | peek()      | O(1)         | O(1)       |
 * | pop()       | O(log n)     | O(log n)   |
 * | push(value) | O(1)         | O(log n)   |
 *
 * @example
 * ```ts
 * import {
 *   ascend,
 *   BinaryHeap,
 *   descend,
 * } from "https://deno.land/std@$STD_VERSION/collections/binary_heap.ts";
 * import { assertEquals } from "https://deno.land/std@$STD_VERSION/testing/asserts.ts";
 *
 * const maxHeap = new BinaryHeap<number>();
 * maxHeap.push(4, 1, 3, 5, 2);
 * assertEquals(maxHeap.peek(), 5);
 * assertEquals(maxHeap.pop(), 5);
 * assertEquals([...maxHeap], [4, 3, 2, 1]);
 * assertEquals([...maxHeap], []);
 *
 * const minHeap = new BinaryHeap<number>(ascend);
 * minHeap.push(4, 1, 3, 5, 2);
 * assertEquals(minHeap.peek(), 1);
 * assertEquals(minHeap.pop(), 1);
 * assertEquals([...minHeap], [2, 3, 4, 5]);
 * assertEquals([...minHeap], []);
 *
 * const words = new BinaryHeap<string>((a, b) => descend(a.length, b.length));
 * words.push("truck", "car", "helicopter", "tank");
 * assertEquals(words.peek(), "helicopter");
 * assertEquals(words.pop(), "helicopter");
 * assertEquals([...words], ["truck", "tank", "car"]);
 * assertEquals([...words], []);
 * ```
 */
export class BinaryHeap {
    constructor(compare = descend) {
        Object.defineProperty(this, "compare", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: compare
        });
        _BinaryHeap_data.set(this, []);
    }
    /** Returns the underlying cloned array in arbitrary order without sorting */
    toArray() {
        return Array.from(__classPrivateFieldGet(this, _BinaryHeap_data, "f"));
    }
    static from(collection, options) {
        let result;
        let unmappedValues = [];
        if (collection instanceof BinaryHeap) {
            result = new BinaryHeap(options?.compare ?? collection.compare);
            if (options?.compare || options?.map) {
                unmappedValues = __classPrivateFieldGet(collection, _BinaryHeap_data, "f");
            }
            else {
                __classPrivateFieldSet(result, _BinaryHeap_data, Array.from(__classPrivateFieldGet(collection, _BinaryHeap_data, "f")), "f");
            }
        }
        else {
            result = options?.compare
                ? new BinaryHeap(options.compare)
                : new BinaryHeap();
            unmappedValues = collection;
        }
        const values = options?.map
            ? Array.from(unmappedValues, options.map, options.thisArg)
            : unmappedValues;
        result.push(...values);
        return result;
    }
    /** The amount of values stored in the binary heap. */
    get length() {
        return __classPrivateFieldGet(this, _BinaryHeap_data, "f").length;
    }
    /** Returns the greatest value in the binary heap, or undefined if it is empty. */
    peek() {
        return __classPrivateFieldGet(this, _BinaryHeap_data, "f")[0];
    }
    /** Removes the greatest value from the binary heap and returns it, or null if it is empty. */
    pop() {
        const size = __classPrivateFieldGet(this, _BinaryHeap_data, "f").length - 1;
        swap(__classPrivateFieldGet(this, _BinaryHeap_data, "f"), 0, size);
        let parent = 0;
        let right = 2 * (parent + 1);
        let left = right - 1;
        while (left < size) {
            const greatestChild = right === size || this.compare(__classPrivateFieldGet(this, _BinaryHeap_data, "f")[left], __classPrivateFieldGet(this, _BinaryHeap_data, "f")[right]) <= 0
                ? left
                : right;
            if (this.compare(__classPrivateFieldGet(this, _BinaryHeap_data, "f")[greatestChild], __classPrivateFieldGet(this, _BinaryHeap_data, "f")[parent]) < 0) {
                swap(__classPrivateFieldGet(this, _BinaryHeap_data, "f"), parent, greatestChild);
                parent = greatestChild;
            }
            else {
                break;
            }
            right = 2 * (parent + 1);
            left = right - 1;
        }
        return __classPrivateFieldGet(this, _BinaryHeap_data, "f").pop();
    }
    /** Adds values to the binary heap. */
    push(...values) {
        for (const value of values) {
            let index = __classPrivateFieldGet(this, _BinaryHeap_data, "f").length;
            let parent = getParentIndex(index);
            __classPrivateFieldGet(this, _BinaryHeap_data, "f").push(value);
            while (index !== 0 && this.compare(__classPrivateFieldGet(this, _BinaryHeap_data, "f")[index], __classPrivateFieldGet(this, _BinaryHeap_data, "f")[parent]) < 0) {
                swap(__classPrivateFieldGet(this, _BinaryHeap_data, "f"), parent, index);
                index = parent;
                parent = getParentIndex(index);
            }
        }
        return __classPrivateFieldGet(this, _BinaryHeap_data, "f").length;
    }
    /** Removes all values from the binary heap. */
    clear() {
        __classPrivateFieldSet(this, _BinaryHeap_data, [], "f");
    }
    /** Checks if the binary heap is empty. */
    isEmpty() {
        return __classPrivateFieldGet(this, _BinaryHeap_data, "f").length === 0;
    }
    /** Returns an iterator for retrieving and removing values from the binary heap. */
    *drain() {
        while (!this.isEmpty()) {
            yield this.pop();
        }
    }
    *[(_BinaryHeap_data = new WeakMap(), Symbol.iterator)]() {
        yield* this.drain();
    }
}
